<?php

use function apexl\io\env;

return [
    'default_vault' => env('VAULT_DEFAULT_VAULT', 'default'),
    "databases" => [
        "default" => [
            "driver" => env('DB_DEFAULT_DRIVER', "mysql"),
            "operator" => env('DB_DEFAULT_OPERATOR', "pdo"),
            "dbhost" => env('DB_DEFAULT_HOST', "db"),
            "port" => env('DB_DEFAULT_PORT', "3306"),
            "dbname" => env('DB_DEFAULT_NAME', "db"),
            "dbuser" => env('DB_DEFAULT_USER', "db"),
            "dbpass" => env('DB_DEFAULT_PASS', "db"),
        ],
        "test" => [
            "driver" => env('DB_TEST_DRIVER', "mysql"),
            "operator" => env('DB_TEST_OPERATOR', "pdo"),
            "dbhost" => env('DB_TEST_HOST', "db"),
            "port" => env('DB_TEST_PORT', "3306"),
            "dbname" => env('DB_TEST_NAME', "test"),
            "dbuser" => env('DB_TEST_USER', "db"),
            "dbpass" => env('DB_TEST_PASS', "db"),
        ],
    ]
];
